# -*- encoding:utf-8 -*-

# 経理システム (仮)
# Copyright (C) 2007 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/


require 'validating_base'

class SearchBox < ValidatingBase
  attr_accessor :bank_name, :branch_name
  
  def validate
    errors.add 'branch_name', 'は必須です。' if branch_name.to_s.strip == ''
  end
end

# 日本国内の銀行マスタ
# 銀行と支店の組
class BanksController < ApplicationController
  before_filter :login_required
  
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  def index
  end
  
  def search
    @search = SearchBox.new params[:search]
    @search.validate
    if !@search.errors.empty?
      render :action => 'index'
      return 
    end
    
    text_br = '%' + Bank.zen2han(@search.branch_name.strip) + '%'
    br_zen = '%' + norm(@search.branch_name.strip) + '%'
    if @search.bank_name.to_s.strip != ""
      text_b = '%' + Bank.zen2han(@search.bank_name.strip) + '%'
      b_zen = '%' + norm(@search.bank_name.strip) + '%'
      @s = BankBranch.find :all,
             :select => 'bank_branches.*', 
             :joins => 'JOIN banks ON bank_branches.bank_id = banks.id',
             :conditions => ['(bank_branches.name LIKE ? OR bank_branches.kana_name LIKE ?) AND (banks.name LIKE ? OR banks.kana_name LIKE ?)', br_zen, text_br, b_zen, text_b],
             :limit => 31
    else
      @s = BankBranch.find :all,
               :conditions => ['name LIKE ? OR kana_name LIKE ?', br_zen, text_br],
               :limit => 31
    end
  end
  
  def list
    #@bank_pages, @banks = paginate :banks, :per_page => 10
    @banks = Bank.paginate :page => params[:page], :per_page => 20
  end

  def show
    @branch = BankBranch.find(params[:id])
  end

  def new
    @bank = Bank.new
  end

  def create
    @bank = Bank.new(params[:bank])
    if @bank.save
      flash[:notice] = 'Bank was successfully created.'
      redirect_to :action => 'list'
    else
      render :action => 'new'
    end
  end

  def edit
    @bank = Bank.find(params[:id])
  end

  def update
    @bank = Bank.find(params[:id])
    if @bank.update_attributes(params[:bank])
      flash[:notice] = 'Bank was successfully updated.'
      redirect_to :action => 'show', :id => @bank
    else
      render :action => 'edit'
    end
  end

  def destroy
    Bank.find(params[:id]).destroy
    redirect_to :action => 'list'
  end
end
