# -*- encoding:utf-8 -*-

# 経理システム (仮)
# Copyright (C) 2007 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/


require 'time'
require 'date'
require 'digest/sha1'

require 'cube'
require 'map_table'
require 'view_term'

# 総勘定元帳
class GeneralLedgerController < ApplicationController
  before_filter :login_required
  
  def index
    @cost_centers = ControlDivision.find :all,
             :select => 'DISTINCT control_divisions.*',
             :joins => "JOIN controllers_divisions ON control_divisions.id = controllers_divisions.control_division_id",
             :conditions => ["controllers_divisions.division_id IN (SELECT division_id FROM users_divisions WHERE user_id = ?)", current_user.id]
    today = Date.today
    @term = ViewTerm.new(:start_year => today.year, :start_mon => today.mon,
                         :end_year => today.year, :end_mon => today.mon)
    end
  
  # 部門の予算・実績対比表の出力
  def comparison
    @term = ViewTerm.new params[:term]
    @term.validate
    @kubuns = Kubun.find params[:kubun]
    if !@term.errors.empty?
      render :action => 'index'
      return
    end

    @cdiv = current_user.default_division.default_controller
    raise if !@cdiv
    
    gl = GeneralLedger.find :all,
              :select => 'date, kubun_id, division_id, account_title_id, sum(amount) as amount',
              :conditions => [<<EOF, @term.sd, @term.ed, @cdiv.id],
(kubun_id IN (#{params[:kubun].join(',')})) AND
(date BETWEEN ? AND ?) AND
division_id IN (SELECT division_id FROM controllers_divisions
                                   WHERE control_division_id = ?)
EOF
              :group => 'kubun_id, division_id, account_title_id, date'
    
    # キューブをviewstateとして作る
    # (年月, 元帳区分, 会計部門, 勘定科目) => 金額
    cube = Cube.new 4
    gl.each {|r|
      cube.add [year_mon(r.date), r.kubun_id, r.division_id, r.account_title.account_code], r.amount
    }
    @table = MapTable.new
    axis1 = Kubun.find cube.axis(1)
    axis3 = cube.axis(3).sort {|x, y| x <=> y}
    
    axis3.each_with_index {|accode, i|
      ac = AccountTitle.find_by_account_code(accode)
      @table.map 0, 2 + i, "#{ac.account_code}#{ac.main_name}", :th => true
    }
    cube.axis(0).each_with_index {|key0, i0|
      x = 1 + i0 * axis1.size
      @table.map x, 0, key0, :colspan => axis1.size
      axis1.each_with_index {|key1, i1|
        @table.map(x + i1, 1, key1.long_name)
        axis3.each_with_index {|key3, i3|
          amount = cube[key0, key1.id, nil, key3]
          if amount != 0
            @table.map x + i1, 2 + i3, "<a href=\"/general_ledger/ledger?ym=#{key0}&amp;code=#{key3}&amp;cc=#{@cdiv.id}&amp;kubun=#{key1.id}\">#{format_amount(amount, 0)}</a>"
          else
            @table.map x + i1, 2 + i3, '-'
          end
        }
      }
    }
    @cube_ = cube
  end
  
  def year_mon d
    return sprintf("%04d%02d", d.year, d.mon)
  end
  private :year_mon

  # 部門別予算・実績対比表 (経理用、全部門)
  def view
  end

  def view2
    @term = ViewTerm.new params[:term]
    @term.validate
    @kubuns = Kubun.find params[:kubun]
    @controllers = ControlDivision.find :all, :conditions => ['parent_id = ?', @term.ctr]
    
    @vs = Digest::SHA1.hexdigest Time.now.to_f.to_s
    session[@vs] = {
      :term => @term,
      :kubuns => params[:kubun],
      :root => @term.ctr }
  end
  
  # 元帳 (画面表示)
  def ledger
    /(20[0-9][0-9])([01][0-9])/ =~ params[:ym]; @year = $1.to_i; @mon = $2.to_i
    st = Date.new(@year, @mon, 1); ed = (st >> 1) - 1
    @accode = params[:code].to_i
    @kubun = Kubun.find params[:kubun]
    @cost_center = ControlDivision.find params[:cc]
    @gl = GeneralLedger.find :all,
                :select => 'general_ledger.*',
                :joins => 'JOIN account_titles ON general_ledger.account_title_id = account_titles.id',
                :conditions => [<<EOF, @kubun.id, st, ed, @accode, @cost_center.id],
kubun_id = ? AND
(date BETWEEN ? AND ?) AND
account_titles.account_code = ? AND
division_id IN (
    SELECT division_id FROM controllers_divisions WHERE control_division_id = ?)
EOF
                :order => "date, slip_number"
    end
  
  # 元帳のダウンロード
  def item_ledger
    term = ViewTerm.new(params[:term])
    raise if !term.sd || !term.ed || term.sd > term.ed
    
    actual_kubun = Kubun.find_by_name('actual')
    item = AccountTitle.find(params[:item][:id])
    @gl = GeneralLedger.find(:all,
          :conditions => ['kubun_id = ? and (date between ? and ?) and account_title_id = ?',
             actual_kubun.id, term.sd, term.ed, item.id],
          :include => :account_title)

    # 出力
    headers["Content-Type"] = "application/vnd.ms-excel; charset=UTF-8"
    headers["Content-Disposition"] = "attachment;\n  filename*=iso-2022-jp'ja'" +
                                     NKF.nkf("-m0 -j -W", "科目別元帳.xls")
    render :action => 'item_ledger', :layout => false
  end
  
  # B/S・P/L実績
  def download_bs
    vs_key = params[:vs]
    term = session[vs_key][:term]
    root = ControlDivision.find session[vs_key][:root]
    actual_kubun = Kubun.find_by_name('actual'); raise if !actual_kubun
    
    gl = GeneralLedger.find :all,
             :select => 'general_ledger.*',
             :conditions => ["kubun_id = ? AND (date BETWEEN ? AND ?)",
                             actual_kubun.id, term.sd, term.ed]
    dl_output(root, nil, term, gl) {|r|
      if r.account_title.account_code > 999
        true
      else
        false
      end
    }
  end
  
  # 予算・実績対比表のダウンロード
  def download
    vs_key = params[:vs]
    term = session[vs_key][:term]
    root = ControlDivision.find session[vs_key][:root]
    kubuns = session[vs_key][:kubuns]
    control_division = ControlDivision.find(params[:division]) if params[:division]
    
    if control_division
      cond = [<<EOF, term.sd, term.ed, control_division.id]
(kubun_id IN (#{kubuns.join(',')})) AND
(date BETWEEN ? AND ?) AND
division_id IN (SELECT division_id FROM controllers_divisions
                                   WHERE control_division_id = ?)
EOF
    else
      cond = [<<EOF, term.sd, term.ed]
(kubun_id IN (#{kubuns.join(',')})) AND (date BETWEEN ? AND ?)
EOF
    end

    gl = GeneralLedger.find :all,
                            :select => "general_ledger.*",
                            :conditions => cond
    not_output = []
    # 722, 801, 802, 803, 804, 805, 806, 807, 808, 809, 810, 831, 832, 833, 834, 835, 836, 837, 838, 905, 913, 915]
    # TODO: 決めうちしてる。fixme
    
    dl_output(root, control_division, term, gl) {|r|
      if not_output.include?(r.account_title.account_code) ||
        r.account_title.account_code < 601 || r.account_title.account_code > 999 # TODO:fixme
        true
      else
        false
      end
    }
  end
  
  private
  def dl_output root, control_division, term, gl
    data = <<EOF
<html xmlns:o="urn:schemas-microsoft-com:office:office"
      xmlns:x="urn:schemas-microsoft-com:office:excel"
      xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja" lang="ja"<
<head>
  <meta http-equiv='Content-Type' content='text/html;charset=utf-8'>
  <meta name='ProgId' content='Excel.Sheet'>
  <xml>
    <o:DocumentProperties>
      <o:Created>#{Time.now.iso8601}</o:Created>
      <o:Company>●</o:Company>
    </o:DocumentProperties>
  </xml>
  <style>
    @page {
        margin:.59in .39in .39in .39in;
        mso-header-margin:.51in;
        mso-footer-margin:.31in;
        mso-page-orientation:landscape;}
    td {
      font-size:9.0pt; }
  </style>
</head>
<body>
    <h1>#{control_division ? control_division.name : nil} #{term.sd}-#{term.ed} 実績</h1>
<table border="0" cellpadding="0" cellspacing="0" width="100%"
    style='border-collapse:collapse;table-layout:fixed;'>
<tr>
  <th>伝票日付</th>
  <th>年月</th>
  <th>区分</th>
  <th>伝票NO</th>
  <th>作成日</th>
  <th>作成者コード</th>
  <th>作成者名</th>
  <th>科目コード</th>
  <th>補助コード</th>
  <th>科目名</th>
  <th>科目名2</th>
  <th>部門コード</th>
  <th>部門名</th>
  <th>金額</th>
  <th>摘要</th>
</tr>
EOF
    
    gl.each {|r|
      next if block_given? && yield(r) 

      data << <<EOF
<tr>
  <td>#{r.date}</td>
  <td>#{sprintf("%04d%02d", r.date.year, r.date.mon)}</td>
  <td>#{Kubun.find(r.kubun_id).long_name}</td>
  <td>#{r.has_attribute?('slip_number') ? r.slip_number : ''}</td>
  <td>#{r.has_attribute?('created_at') && r.created_at ? r.created_at.strftime("%Y/%m/%d") : ''}</td>
  <td>#{r.has_attribute?('created_by') && r.created_by ? r.created_by.worker_code : nil}</td>
  <td>#{r.has_attribute?('created_by') && r.created_by ? r.created_by.name : nil}</td>
  <td>#{r.account_title.account_code}</td>
  <td>#{r.account_title.suppl_code}</td>
  <td>#{r.account_title.name}</td>
  <td>#{r.account_title.account_code}#{r.account_title.main_name}
  <td>#{control_division ? r.division.division_code : nil}</td>
  <td>#{control_division ? r.division.name :
            (r.division ? r.division.controller(root).name : '')}</td>
  <td>#{r.amount}</td>
  <td>#{r.has_attribute?('remarks') ? r.remarks : ''}</td>
</tr>
EOF
    }
    data << "</table></body></html>"
    
    send_data data, :type => "application/vnd.ms-excel; charset=UTF-8", :disposition => %Q(attachment; filename="#{
    sprintf("%04d%02d", term.ed.year, term.ed.mon)}#{control_division ? control_division.name : 'all'}.xls")
  end
end

