# -*- encoding:utf-8 -*-

# 経理システム (仮)
# Copyright (C) 2007 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/


# 経理だけが使う関係
class KeiriController < ApplicationController
  before_filter :login_required
  before_filter :role_keiri_required
  
  def role_keiri_required
    if !current_user.role_keiri?
      redirect_to '/'
    end
  end
  private :role_keiri_required
  
  def index
  end
  
  # 外貨建の金額を円に換算
  def get_yen_amount entry_day, amount, currency
    cr = CurrencyRate.find :first,
                 :select => 'MAX(date) as date',
                 :conditions => ['currency_id = ? AND date <= ?', currency.id, entry_day]
    raise if !cr
    cr = CurrencyRate.find :first, :conditions => ['currency_id = ? AND date = ?',
                                              currency.id, cr.date]
    return [(Float(amount) * cr.rate / 10 ** currency.exponent + 0.5).floor, cr.rate]
  end
  private :get_yen_amount
  
  # 仕訳確認画面
  def export
    invoices = Invoice.find :all, :conditions => 'state = 10',
                                  :order => 'buy_day'
    @size = invoices.size
    fixed_day = CompanySetting.find(:first).fixed_day
    @entries = []
    invoices.each {|inv|
      entry_day = [inv.buy_day, fixed_day + 1].max
      @entries += make_entry(inv, entry_day)
    }
    session[:entries] = invoices.collect {|x| x.id}
  end
  
  # 実際の出力
  def export2
    inv_ids = session[:entries]
    fixed_day = CompanySetting.find(:first).fixed_day
    @entries = []
    inv_ids.each {|inv_id|
      inv = Invoice.find inv_id
      entry_day = [inv.buy_day, fixed_day + 1].max
      @entries += make_entry(inv, entry_day)
    }
    headers["Content-Type"] = "application/vnd.ms-excel; charset=UTF-8"
    headers["Content-Disposition"] = "attachment;\n  filename*=iso-2022-jp'ja'" +
                                     NKF.nkf("-m0 -j -W", "purchase.csv")
    render :action => 'export2', :layout => false

    # 連動済みとしてマークする。
    Invoice.transaction {
      inv_ids.each {|inv_id|
        inv = Invoice.find inv_id
        inv.state = 20
        inv.journal_day = [inv.buy_day, fixed_day + 1].max
        inv.save!
      }
    }
  end
  
  def make_entry inv, entry_day
    result = []
    inv.invoice_details.each {|detail|
      amount, rate = get_yen_amount(entry_day, detail.amount, inv.currency)
      line = {
        :entry_day => entry_day,
        :entry_no => 10000 + inv.id,  # TODO:決め打ち。fixme
        :account_title => detail.account_title,
        :division => detail.division,
        :payable_code => detail.account_title.get_payable_code,
        :bank_account => (inv.bank_account ? inv.bank_account.zaimu_ac : inv.partner.zaimu_id),
        :amount => amount,
        :vat => detail.account_title.get_real_vat_code(detail.vat_code), # for compat.
        :due_day => (inv.bank_account ? inv.due_day : nil),
        :created_by => inv.created_by,
      }
      if inv.currency.code == "JPY"  # TODO:fixme
        line[:remarks] = [detail.division.name, inv.partner.name,
                            detail.remarks].join('/')
      else
        line[:remarks] = [detail.division.name, inv.partner.name,
          "#{detail.remarks} #{format_amount(detail.amount, inv.currency.exponent)}#{inv.currency.code}@#{rate}"].join('/')
      end
      result << line
    }
    return result
  end
  private :make_entry

  # 総勘定元帳 (仕訳) のインポート画面
  def import_gl
    require 'csv'

    @fname = '/shiwake.csv'
    if FileTest.exist?(IMPORT_DATA_DIR + @fname)
      data = sjis2utf File.read(IMPORT_DATA_DIR + @fname)
      @head = []
      
      idx = 0
      CSV::Reader.parse(data) {|line|
        idx += 1; next if idx == 1
        break if idx == 10
        @head << line
      }
    end
  end
  
  SCRIPT_DIR = File.dirname(__FILE__) + '/../../script'
  def do_import_gl
    `#{KeiriController::SCRIPT_DIR}/run_import_gl.rb`
    redirect_to :action => 'import_gl_log'
  end
  
  # ログを表示
  def import_gl_log
    @out = File.read KeiriController::SCRIPT_DIR + '/import_gl_out.log'
    @err = File.read KeiriController::SCRIPT_DIR + '/import_gl_err.log'
  end
  
  # 取引先（支払先）のインポート画面
  def import_partners
    `#{KeiriController::SCRIPT_DIR}/run_import_partners.rb`
    redirect_to :action => 'import_partners_log'
  end
  
  # ログ表示
  def import_partners_log
    @out = File.read KeiriController::SCRIPT_DIR + '/import_partners_out.log'
    @err = File.read KeiriController::SCRIPT_DIR + '/import_partners_err.log'
  end
end
