# -*- encoding:utf-8 -*-

# 経理システム (仮)
# Copyright (C) 2007 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/


require 'ostruct'  # DEBUG


# 売上側の請求書 (売掛金管理)
class SalesInvoicesController < ApplicationController
  append_after_filter :make_pdf, :only => :print_invoice
  
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  def index
    list
    render :action => 'list'
  end
  
  SCRIPT_DIR = File.dirname(__FILE__) + '/../../script'
  # PDFに変換する
  def make_pdf
    File.open('/tmp/q', 'w') {|f| f.write response.body }
    headers['Content-Type'] = 'application/pdf'
    ENV['JAVA_HOME'] = '/usr/java/latest'
    `/opt/fop-0.94/fop -c #{SCRIPT_DIR}/fop-conf.xml /tmp/q -pdf /tmp/r`
    response.body = File.read('/tmp/r')
  end
  private :make_pdf
  
  # 作った請求書を出力
  def print_invoice
    @company = OpenStruct.new(
              :name => '株式会社 テスト商事',
              :url => 'www.yahoo!!!.com',
              :address => '福岡市博多区2丁目2-1 FooBarビル',
              :tel => '092-555-1111',
              :zip => '812-xxxx',
              :finance_dept => '財務経理部長　田中　太郎',
              :person => 'ほげ山ほげ太',
        :banks => [
          OpenStruct.new(:bank_name => '三井住友銀行',
                       :branch_name => '天神支店',
                       :kubun => '普通',
                      :account => '1239876')
        ]    )
    @invoice = SalesInvoice.find params[:id]
    @today = Date.today
    render :layout => false
  end
  
  def list
    @sales_invoices = SalesInvoice.paginate :page => params[:page], :per_page => 20
  end

  def show
    @invoice = SalesInvoice.find(params[:id])
  end
  
  # 新規作成画面
  def new
    @partner = Partner.find params[:partner]
    @invoice = SalesInvoice.new
    @invoice.contact_name1 = @partner.name
  end
  
  # 明細行のレコードを作る
  def create_details new_exp
    result = []
    (0..8).each {|i| # magic
      next if (pa = params["detail#{i}"])[:amount].to_f == 0.0
      
      detail = SiDetail.new(pa)
      detail.attributes = {
        :sales_invoice => @invoice,
        :unit_price => pa[:unit_price].blank? ? nil : (pa[:unit_price].to_f * (10 ** new_exp) + 0.5).floor,
        :line_no => i + 1
      }
      if detail.unit_price.to_i != 0 && detail.qty.to_i != 0
        detail.amount = detail.unit_price * detail.qty
      else
        detail.amount = (pa[:amount].to_f * (10 ** new_exp) + 0.5).floor
      end
      result << detail
    }
    return result
  end
  private :create_details

  # 作成
  def create
    @partner = Partner.find params[:partner]
    @invoice = SalesInvoice.new params[:invoice]
    @invoice.attributes = {
      :partner => @partner,
      :division => current_user.default_division,
      :created_by => current_user,
      :state => 0,
    }
    
    SalesInvoice.transaction {
      (details = create_details(@invoice.currency.exponent)).each {|r|
        @invoice.si_details << r }
      banks = OurBank.find(params[:ba])
      if @invoice.save
        details.each {|r| r.save! }
        banks.each {|r|
          b = SiBank.new :sales_invoice => @invoice,
                         :our_bank => r
          b.save!
        }
        
        flash[:notice] = 'SalesInvoice was successfully created.'
        redirect_to :action => 'show', :id => @invoice
      else
        render :action => 'new', :partner => @partner
      end
    } # of transaction
  end
  
  # 編集画面
  def edit
    @invoice = SalesInvoice.find(params[:id])
    @partner = @invoice.partner
  end

  def update
    @sales_invoice = SalesInvoice.find(params[:id])
    if @sales_invoice.update_attributes(params[:sales_invoice])
      flash[:notice] = 'SalesInvoice was successfully updated.'
      redirect_to :action => 'show', :id => @sales_invoice
    else
      render :action => 'edit'
    end
  end

  def destroy
    SalesInvoice.find(params[:id]).destroy
    redirect_to :action => 'list'
  end
end
