
require 'digest/sha1'

# ユーザ (社員)
class User < ActiveRecord::Base
  belongs_to :division  # TODO:廃止、users_divisions経由で。
  has_many :users_divisions
  has_many :divisions, :through => :users_divisions
  
  # Virtual attribute for the unencrypted password
  attr_accessor :password
  attr_accessor :password_confirmation
  before_save :encrypt_password

  validates_presence_of :worker_code
  validates_uniqueness_of :worker_code, :message => "が重複しています。"
  validates_presence_of :name
  validates_presence_of :kana_name

  # @override
  def validate
    errors.add "worker_code", "は0以外でなければなりません。" if worker_code == 0
    if !login.blank?
      errors.add 'email', 'がありません。' if !email || email.strip == ''
      if !password.blank?
        errors.add 'password', 'が短すぎます。' if password.length < 3
        errors.add 'password_confirmation', 'が一致していません。' if password != password_confirmation
      end
    end
  end
  
  # @override
  def validate_on_create
    if !login.blank?
      errors.add 'login', 'が重複しています。' if User.find_by_login(login)
      errors.add 'email', 'が重複しています。' if User.find_by_email(email)
      errors.add 'password', 'がありません。' if password.blank?
    end
  end

  # @override
  def validate_on_update
    if !login.blank?
      if User.find(:first, :conditions => ['login = ? AND id <> ?', login, id])
        errors.add 'login', 'が重複しています。'
      end
      if User.find(:first, :conditions => ['email = ? AND id <> ?', email, id])
        errors.add 'email', 'が重複しています。' 
      end
    end
  end
  
  # 本社経理だけ特別扱い
  def role_keiri?
    divisions.each {|d|
      return true if d.role && d.role.keiri != 0
    }
    # compat.
    return division && division.role && division.role.keiri != 0
  end
  
  # 主な所属部門
  def default_division
    # TODO: まじめに作ること
    divisions.first || division  # compat.
  end
  

  ###########################################################################
  # from acts_as_authenticated

  # Authenticates a user by their login name and unencrypted password.
  # Returns the user or nil.
  def self.authenticate(login, password)
    u = find_by_login(login) # need to get the salt
    u && u.authenticated?(password) ? u : nil
  end

  # Encrypts some data with the salt.
  def self.encrypt(password, salt)
    Digest::SHA1.hexdigest("--#{salt}--#{password}--")
  end

  # Encrypts the password with the user salt
  def encrypt(password)
    self.class.encrypt(password, salt)
  end

  def authenticated?(password)
    crypted_password == encrypt(password)
  end

  def remember_token?
    remember_token_expires_at && Time.now.utc < remember_token_expires_at 
  end

  # These create and unset the fields required for remembering users between browser closes
  def remember_me
    self.remember_token_expires_at = 2.weeks.from_now.utc
    self.remember_token            = encrypt("#{email}--#{remember_token_expires_at}")
    save(false)
  end

  def forget_me
    self.remember_token_expires_at = nil
    self.remember_token            = nil
    save(false)
  end

  protected
  # before filter 
  def encrypt_password
    return if password.blank?
    self.salt = Digest::SHA1.hexdigest("--#{Time.now.to_s}--#{login}--") if new_record?
    self.crypted_password = encrypt(password)
  end

  def password_required?
    crypted_password.blank? || !password.blank?
  end
  
  # from acts_as_authenticated
  ###########################################################################
end
